@@ -1,9 +1,35 @@
 #
 # Module providing the `Process` class which emulates `threading.Thread`
 #
-# billiard/process.py
+# multiprocessing/process.py
 #
-# Copyright (c) 2006-2008, R Oudkerk --- see COPYING.txt
+# Copyright (c) 2006-2008, R Oudkerk
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. Neither the name of author nor the names of any contributors may be
+#    used to endorse or promote products derived from this software
+#    without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+# SUCH DAMAGE.
 #
 from __future__ import absolute_import
 
@@ -20,10 +46,10 @@ import itertools
 import binascii
 
 from .compat import bytes
-
-#
-#
-#
+try:
+    from _weakrefset import WeakSet
+except ImportError:
+    WeakSet = None  # noqa
 
 try:
     ORIGINAL_DIR = os.path.abspath(os.getcwd())
@@ -34,12 +60,14 @@ except OSError:
 # Public functions
 #
 
+
 def current_process():
     '''
     Return process object representing the current process
     '''
     return _current_process
 
+
 def active_children():
     '''
     Return list of process objects corresponding to live child processes
@@ -47,9 +75,6 @@ def active_children():
     _cleanup()
     return list(_current_process._children)
 
-#
-#
-#
 
 def _cleanup():
     # check for processes which have finished
@@ -57,9 +82,6 @@ def _cleanup():
         if p._popen.poll() is not None:
             _current_process._children.discard(p)
 
-#
-# The `Process` class
-#
 
 class Process(object):
     '''
@@ -69,12 +91,16 @@ class Process(object):
     '''
     _Popen = None
 
-    def __init__(self, group=None, target=None, name=None, args=(), kwargs={}):
+    def __init__(self, group=None, target=None, name=None, args=(), kwargs={},
+            daemon=None, **_kw):
         assert group is None, 'group argument must be None for now'
         count = _current_process._counter.next()
         self._identity = _current_process._identity + (count,)
         self._authkey = _current_process._authkey
-        self._daemonic = _current_process._daemonic
+        if daemon is not None:
+            self._daemonic = daemon
+        else:
+            self._daemonic = _current_process._daemonic
         self._tempdir = _current_process._tempdir
         self._semprefix = _current_process._semprefix
         self._unlinkfd = _current_process._unlinkfd
@@ -85,6 +111,8 @@ class Process(object):
         self._kwargs = dict(kwargs)
         self._name = name or type(self).__name__ + '-' + \
                      ':'.join(str(i) for i in self._identity)
+        if _dangling is not None:
+            _dangling.add(self)
 
     def run(self):
         '''
@@ -108,6 +136,7 @@ class Process(object):
         else:
             from .forking import Popen
         self._popen = Popen(self)
+        self._sentinel = self._popen.sentinel
         _current_process._children.add(self)
 
     def terminate(self):
@@ -182,6 +211,17 @@ class Process(object):
 
     pid = ident
 
+    @property
+    def sentinel(self):
+        '''
+        Return a file descriptor (Unix) or handle (Windows) suitable for
+        waiting for process termination.
+        '''
+        try:
+            return self._sentinel
+        except AttributeError:
+            raise ValueError("process not started")
+
     def __repr__(self):
         if self is _current_process:
             status = 'started'
@@ -213,14 +253,21 @@ class Process(object):
         try:
             self._children = set()
             self._counter = itertools.count(1)
-            try:
-                sys.stdin.close()
-                sys.stdin = open(os.devnull)
-            except (OSError, ValueError):
-                pass
+            if sys.stdin is not None:
+                try:
+                    sys.stdin.close()
+                    sys.stdin = open(os.devnull)
+                except (OSError, ValueError):
+                    pass
+            old_process = _current_process
             _current_process = self
-            util._finalizer_registry.clear()
-            util._run_after_forkers()
+            try:
+                util._finalizer_registry.clear()
+                util._run_after_forkers()
+            finally:
+                # delay finalization of the old process object until after
+                # _run_after_forkers() is executed
+                del old_process
             util.info('child process calling self.run()')
             try:
                 self.run()
@@ -234,36 +281,40 @@ class Process(object):
                 exitcode = e.args[0]
             else:
                 sys.stderr.write(e.args[0] + '\n')
-                sys.stderr.flush()
                 exitcode = 1
         except:
             exitcode = 1
-            import traceback
-            sys.stderr.write('Process %s:\n' % self.name)
+            if not util.error('Process %s', self.name, exc_info=True):
+                import traceback
+                sys.stderr.write('Process %s:\n' % self.name)
+                traceback.print_exc()
+        finally:
+            util.info('process exiting with exitcode %d', exitcode)
+            sys.stdout.flush()
             sys.stderr.flush()
-            traceback.print_exc()
-
-        util.info('process exiting with exitcode %d' % exitcode)
         return exitcode
 
 #
 # We subclass bytes to avoid accidental transmission of auth keys over network
 #
 
+
 class AuthenticationString(bytes):
+
     def __reduce__(self):
         from .forking import Popen
+
         if not Popen.thread_is_spawning():
             raise TypeError(
                 'Pickling an AuthenticationString object is '
-                'disallowed for security reasons'
-                )
+                'disallowed for security reasons')
         return AuthenticationString, (bytes(self),)
 
 #
 # Create object representing the main process
 #
 
+
 class _MainProcess(Process):
 
     def __init__(self):
@@ -276,7 +327,8 @@ class _MainProcess(Process):
         self._children = set()
         self._authkey = AuthenticationString(os.urandom(32))
         self._tempdir = None
-        self._semprefix = 'mp-'+binascii.hexlify(os.urandom(4)).decode('ascii')
+        self._semprefix = 'mp-' + binascii.hexlify(
+                                os.urandom(4)).decode('ascii')
         self._unlinkfd = None
 
 _current_process = _MainProcess()
@@ -289,5 +341,7 @@ del _MainProcess
 _exitcode_to_name = {}
 
 for name, signum in signal.__dict__.items():
-    if name[:3]=='SIG' and '_' not in name:
+    if name[:3] == 'SIG' and '_' not in name:
         _exitcode_to_name[-signum] = name
+
+_dangling = WeakSet() if WeakSet is not None else None
