@@ -1,11 +1,38 @@
 #
 # Module implementing queues
 #
-# billiard/queues.py
+# multiprocessing/queues.py
 #
-# Copyright (c) 2006-2008, R Oudkerk --- see COPYING.txt
+# Copyright (c) 2006-2008, R Oudkerk
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. Neither the name of author nor the names of any contributors may be
+#    used to endorse or promote products derived from this software
+#    without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND
+# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
+# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
+# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
+# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
+# SUCH DAMAGE.
 #
 from __future__ import absolute_import
+from __future__ import with_statement
 
 __all__ = ['Queue', 'SimpleQueue', 'JoinableQueue']
 
@@ -14,22 +41,22 @@ import os
 import threading
 import collections
 import time
-import atexit
 import weakref
+import errno
 
 from Queue import Empty, Full
-import _billiard
+
 from . import Pipe
+from ._ext import _billiard
 from .synchronize import Lock, BoundedSemaphore, Semaphore, Condition
-from .util import debug, info, Finalize, register_after_fork
+from .util import debug, error, info, Finalize, register_after_fork
 from .forking import assert_spawning
 
-#
-# Queue type using a pipe, buffer and thread
-#
 
 class Queue(object):
-
+    '''
+    Queue type using a pipe, buffer and thread
+    '''
     def __init__(self, maxsize=0):
         if maxsize <= 0:
             maxsize = _billiard.SemLock.SEM_VALUE_MAX
@@ -42,6 +69,8 @@ class Queue(object):
         else:
             self._wlock = Lock()
         self._sem = BoundedSemaphore(maxsize)
+        # For use by concurrent.futures
+        self._ignore_epipe = False
 
         self._after_fork()
 
@@ -50,11 +79,11 @@ class Queue(object):
 
     def __getstate__(self):
         assert_spawning(self)
-        return (self._maxsize, self._reader, self._writer,
+        return (self._ignore_epipe, self._maxsize, self._reader, self._writer,
                 self._rlock, self._wlock, self._sem, self._opid)
 
     def __setstate__(self, state):
-        (self._maxsize, self._reader, self._writer,
+        (self._ignore_epipe, self._maxsize, self._reader, self._writer,
          self._rlock, self._wlock, self._sem, self._opid) = state
         self._after_fork()
 
@@ -76,24 +105,18 @@ class Queue(object):
         if not self._sem.acquire(block, timeout):
             raise Full
 
-        self._notempty.acquire()
-        try:
+        with self._notempty:
             if self._thread is None:
                 self._start_thread()
             self._buffer.append(obj)
             self._notempty.notify()
-        finally:
-            self._notempty.release()
 
     def get(self, block=True, timeout=None):
         if block and timeout is None:
-            self._rlock.acquire()
-            try:
+            with self._rlock:
                 res = self._recv()
                 self._sem.release()
                 return res
-            finally:
-                self._rlock.release()
 
         else:
             if block:
@@ -101,7 +124,11 @@ class Queue(object):
             if not self._rlock.acquire(block, timeout):
                 raise Empty
             try:
-                if not self._poll(block and (deadline-time.time()) or 0.0):
+                if block:
+                    timeout = deadline - time.time()
+                    if timeout < 0 or not self._poll(timeout):
+                        raise Empty
+                elif not self._poll():
                     raise Empty
                 res = self._recv()
                 self._sem.release()
@@ -110,7 +137,8 @@ class Queue(object):
                 self._rlock.release()
 
     def qsize(self):
-        # Raises NotImplementedError on Mac OSX because of broken sem_getvalue()
+        # Raises NotImplementedError on Mac OSX because
+        # of broken sem_getvalue()
         return self._maxsize - self._sem._semlock._get_value()
 
     def empty(self):
@@ -153,7 +181,7 @@ class Queue(object):
         self._thread = threading.Thread(
             target=Queue._feed,
             args=(self._buffer, self._notempty, self._send,
-                  self._wlock, self._writer.close),
+                  self._wlock, self._writer.close, self._ignore_epipe),
             name='QueueFeederThread'
             )
         self._thread.daemon = True
@@ -196,37 +224,29 @@ class Queue(object):
     @staticmethod
     def _finalize_close(buffer, notempty):
         debug('telling queue thread to quit')
-        notempty.acquire()
-        try:
+        with notempty:
             buffer.append(_sentinel)
             notempty.notify()
-        finally:
-            notempty.release()
 
     @staticmethod
-    def _feed(buffer, notempty, send, writelock, close):
+    def _feed(buffer, notempty, send, writelock, close, ignore_epipe):
         debug('starting thread to feed data to pipe')
         from .util import is_exiting
 
-        nacquire = notempty.acquire
-        nrelease = notempty.release
+        ncond = notempty
         nwait = notempty.wait
         bpopleft = buffer.popleft
         sentinel = _sentinel
         if sys.platform != 'win32':
-            wacquire = writelock.acquire
-            wrelease = writelock.release
+            wlock = writelock
         else:
-            wacquire = None
+            wlock = None
 
         try:
             while 1:
-                nacquire()
-                try:
+                with ncond:
                     if not buffer:
                         nwait()
-                finally:
-                    nrelease()
                 try:
                     while 1:
                         obj = bpopleft()
@@ -235,41 +255,42 @@ class Queue(object):
                             close()
                             return
 
-                        if wacquire is None:
+                        if wlock is None:
                             send(obj)
                         else:
-                            wacquire()
-                            try:
+                            with wlock:
                                 send(obj)
-                            finally:
-                                wrelease()
                 except IndexError:
                     pass
         except Exception, e:
+            if ignore_epipe and getattr(e, 'errno', 0) == errno.EPIPE:
+                return
             # Since this runs in a daemon thread the resources it uses
             # may be become unusable while the process is cleaning up.
             # We ignore errors which happen after the process has
             # started to cleanup.
             try:
                 if is_exiting():
-                    info('error in queue thread: %s', e)
+                    info('error in queue thread: %r', e, exc_info=True)
                 else:
-                    import traceback
-                    traceback.print_exc()
+                    if not error('error in queue thread: %r', e,
+                                 exc_info=True):
+                        import traceback
+                        traceback.print_exc()
             except Exception:
                 pass
 
 _sentinel = object()
 
-#
-# A queue type which also supports join() and task_done() methods
-#
-# Note that if you do not call task_done() for each finished task then
-# eventually the counter's semaphore may overflow causing Bad Things
-# to happen.
-#
 
 class JoinableQueue(Queue):
+    '''
+    A queue type which also supports join() and task_done() methods
+
+    Note that if you do not call task_done() for each finished task then
+    eventually the counter's semaphore may overflow causing Bad Things
+    to happen.
+    '''
 
     def __init__(self, maxsize=0):
         Queue.__init__(self, maxsize)
@@ -288,45 +309,36 @@ class JoinableQueue(Queue):
         if not self._sem.acquire(block, timeout):
             raise Full
 
-        self._notempty.acquire()
-        self._cond.acquire()
-        try:
-            if self._thread is None:
-                self._start_thread()
-            self._buffer.append(obj)
-            self._unfinished_tasks.release()
-            self._notempty.notify()
-        finally:
-            self._cond.release()
-            self._notempty.release()
+        with self._notempty:
+            with self._cond:
+                if self._thread is None:
+                    self._start_thread()
+                self._buffer.append(obj)
+                self._unfinished_tasks.release()
+                self._notempty.notify()
 
     def task_done(self):
-        self._cond.acquire()
-        try:
+        with self._cond:
             if not self._unfinished_tasks.acquire(False):
                 raise ValueError('task_done() called too many times')
             if self._unfinished_tasks._semlock._is_zero():
                 self._cond.notify_all()
-        finally:
-            self._cond.release()
 
     def join(self):
-        self._cond.acquire()
-        try:
+        with self._cond:
             if not self._unfinished_tasks._semlock._is_zero():
                 self._cond.wait()
-        finally:
-            self._cond.release()
 
-#
-# Simplified Queue type -- really just a locked pipe
-#
 
 class SimpleQueue(object):
+    '''
+    Simplified Queue type -- really just a locked pipe
+    '''
 
     def __init__(self):
         self._reader, self._writer = Pipe(duplex=False)
         self._rlock = Lock()
+        self._poll = self._reader.poll
         if sys.platform == 'win32':
             self._wlock = None
         else:
@@ -334,7 +346,7 @@ class SimpleQueue(object):
         self._make_methods()
 
     def empty(self):
-        return not self._reader.poll()
+        return not self._poll()
 
     def __getstate__(self):
         assert_spawning(self)
@@ -346,13 +358,11 @@ class SimpleQueue(object):
 
     def _make_methods(self):
         recv = self._reader.recv
-        racquire, rrelease = self._rlock.acquire, self._rlock.release
+        rlock = self._rlock
+
         def get():
-            racquire()
-            try:
+            with rlock:
                 return recv()
-            finally:
-                rrelease()
         self.get = get
 
         if self._wlock is None:
@@ -360,11 +370,9 @@ class SimpleQueue(object):
             self.put = self._writer.send
         else:
             send = self._writer.send
-            wacquire, wrelease = self._wlock.acquire, self._wlock.release
+            wlock = self._wlock
+
             def put(obj):
-                wacquire()
-                try:
+                with wlock:
                     return send(obj)
-                finally:
-                    wrelease()
             self.put = put
